import string


def ad_from_string(s, slots=(30, 30, 80, 15, 15), sep=None, capitalize=False):
    """Convert string ``s`` to an ad by splitting it into groups of words.
    Each group would have a length of at most the allowed length for that slot.

    If the total length of ``s`` exceeds the total allowed length, all remaining
    characters would be grouped in the last element of the returned list.

    :param s: a string of characters, with no restrictions on length.
    :param slots: an iterable of integers for the maximum lengths for each slot.
    :param sep: by which character to split ``s``.
    :param capitalize: whether or not to capitalize each word after grouping.
        Setting it as False would leave the input string as is.
    :returns text_ad: a list of strings.

    >>> ad_from_string('this is a short ad')
    ['This Is A Short Ad', '', '', '', '', '']

    >>> ad_from_string('this is a longer ad and would take the first two slots')
    ['This Is A Longer Ad And Would', 'Take The First Two Slots', '', '', '', '']
    """

    str_words = s.split(sep=sep)
    text_ad = ['' for x in range(len(slots)+1)]
    counter = 0

    for i, slot in enumerate(slots):
        while counter <= len(str_words) - 1:
            if len(text_ad[i] + str_words[counter]) + 1 > slot:
                break
            text_ad[i] += ' ' + str_words[counter] if text_ad[i] else str_words[counter]
            counter += 1

    text_ad[-1] = (sep.join(str_words[counter:])
                   if sep is not None else ' '.join(str_words[counter:]))

    return [string.capwords(x) if capitalize else x for x in text_ad]



import urllib


def url_utm_ga(url, utm_source, utm_medium=None, utm_campaign=None,
               utm_content=None, utm_term=None):
    """Generate a URL with UTM codes for your campaigns.

    :param url: a valid URL, required
    :param utm_source: the referrer of the traffic (e.g. facebook, twitter), required
    :param utm_medium: marketing medium (e.g. banner, email)
    :param utm_campaign: the name of the campaign (e.g. summer_promo, 20pct_off)
    :param utm_content: ad name / differentiator (e.g. 728x90, mpu, square_banner)
    :param utm_term: search terms bid on (only relevant for search campaigns)
    :return: URL-encoded string for the campaign

    >>> url_utm_ga('mysite.com', utm_source='the source')
    'mysite.com?utm_source=the+source'

    >>> url_utm_ga('mysite.com', utm_source='the source', utm_medium='THE MEDIUM!!',
                   utm_campaign='campaign*name&^%',utm_content='728x90')
    'mysite.com?utm_content=728x90&utm_campaign=campaign%2Aname%26%5E%25&utm_medium=THE+MEDIUM%21%21&utm_source=the+source'
    """
    url += '?'
    params = {k: v for k, v in locals().items() if k != 'url'}
    return url + urllib.parse.urlencode({k: v for k, v in params.items() if v})




"""Top-level package for advertools."""

__author__ = """Elias Dabbas"""
__email__ = 'eliasdabbas@gmail.com'
__version__ = '0.5.0'

from advertools.ad_create import ad_create
from advertools.ad_from_string import ad_from_string
from advertools.extract import extract_mentions, extract_hashtags, extract_emoji
from advertools.kw_generate import *
from advertools.stopwords import stopwords
from advertools.url_builders import url_utm_ga
from advertools.word_frequency import word_frequency
from . import twitter
from .serp import (serp_goog, serp_youtube, SERP_GOOG_VALID_VALS,
                   youtube_channel_details, youtube_video_details,
                   YOUTUBE_VID_CATEGORY_IDS, YOUTUBE_TOPIC_IDS,
                   set_logging_level)



from advertools.word_frequency import word_frequency


sep_list = [None, ' ', '-', '_']

text_list = [
    'one two',
    'one two  three',
    'one-two-three',
    'one_two_three',
    'four five',
    'four five',
    'four six'
]

num_list = [
    100,
    200,
    300,
    400,
    500,
    600,
    700
]

separators = [
    '-',
    '_',
    ' ',
    'f'
]


def test_len_result_one_more_than_len_slots():
    for sep in sep_list:
        result = word_frequency(text_list, num_list, sep=sep)
        if sep is not None:
            assert sep not in result['word']


def test_rm_words_removed():
    result = word_frequency(text_list, num_list, rm_words=['one', 'two'])
    assert 'one' not in result['word']
    assert 'two' not in result['word']


def test_extra_info_not_provided():
    result = word_frequency(text_list, num_list, extra_info=False)
    assert set(result.columns.values) == {'word', 'abs_freq', 'wtd_freq',
                                          'rel_value'}


def test_extra_info_provided():
    result = word_frequency(text_list, num_list, extra_info=True)
    assert set(result.columns.values) == {'word', 'abs_freq', 'abs_perc',
                                          'abs_perc_cum', 'wtd_freq',
                                          'wtd_freq_perc', 'wtd_freq_perc_cum',
                                          'rel_value'}


def test_words_separated_with_given_sep():
    for sep in separators:
        result = word_frequency(text_list, num_list, sep=sep)
        assert sep not in result['word']


def test_works_fine_with_only_stopwords_supplied():
    result = word_frequency(['on'], [3])
    assert result.shape == (0, 4)




from advertools.extract import extract_mentions, extract_hashtags, extract_emoji

mention_posts = ['hello @name', 'email@domain.com', '@oneword', 'hi @nam-e and @name',
               '@first @last', 'an @under_score', 'a @dot.one', 'non latin @مرحبا',
               'other at ＠sign', '@one.@two three', 'number @123text', '@_before @after_']

mention_summary = extract_mentions(mention_posts)

mention_test_keys = ['mentions', 'mentions_flat', 'mention_counts',
                     'mention_freq', 'top_mentions', 'overview']

hashtag_posts = ['i like #blue', 'i like #green and #blue', 'i like all']
hashtag_summary = extract_hashtags(hashtag_posts)
hashtag_test_keys = ['hashtags', 'hashtags_flat', 'hashtag_counts',
                     'hashtag_freq', 'top_hashtags', 'overview']

emoji_posts = ['one smiley 😀', 'one smiley 😀 one wink 😉', 'no emoji']
emoji_summary = extract_emoji(emoji_posts)
emoji_test_keys = ['emoji', 'emoji_text', 'emoji_flat', 'emoji_flat_text',
                   'emoji_counts', 'emoji_freq', 'top_emoji', 'top_emoji_text', 
                   'overview']


def test_mention_result_has_correct_keys():
    assert set(mention_summary.keys()) == set(mention_test_keys)


def test_correct_mentions_extracted():
    assert mention_summary['mentions'] == [['@name'], [], ['@oneword'],
                                           ['@nam', '@name'],['@first', '@last'],
                                           ['@under_score'], ['@dot'], [],
                                           ['＠sign'], ['@one', '@two'],
                                           ['@123text'], ['@_before', '@after_']]


def test_correct_flat_mentions():
    assert mention_summary['mentions_flat'] == ['@name', '@oneword', '@nam',
                                                '@name', '@first', '@last',
                                                '@under_score', '@dot', '＠sign',
                                                '@one', '@two', '@123text',
                                                '@_before', '@after_']


def test_correct_mention_counts():
    assert mention_summary['mention_counts'] == [1, 0, 1, 2, 2, 1, 1, 0, 
                                                 1, 2, 1, 2]


def test_correct_mention_freq():
    assert mention_summary['mention_freq'] == [(0, 2), (1, 6), (2, 4)]


def test_correct_top_mentions():
    assert set(mention_summary['top_mentions']) == set([('@name', 2), ('@oneword', 1),
                                               ('@nam', 1), ('@first', 1),
                                               ('@last', 1), ('@under_score', 1),
                                               ('@dot', 1), ('＠sign', 1),
                                               ('@one', 1), ('@two', 1),
                                               ('@123text', 1),
                                               ('@_before', 1), ('@after_', 1)])


def test_correct_mention_overview():
    mention_overview = mention_summary['overview']
    assert mention_overview['num_posts'] == 12
    assert mention_overview['num_mentions'] == 14
    assert mention_overview['mentions_per_post'] == 14/12
    assert mention_overview['unique_mentions'] == 13


def test_hashtag_result_has_correct_keys():
    assert set(hashtag_summary.keys()) == set(hashtag_test_keys)


def test_correct_hashtags_extracted():
    assert hashtag_summary['hashtags'] == [['#blue'], ['#green', '#blue'], []]


def test_correct_flat_hashtags():
    assert hashtag_summary['hashtags_flat'] == ['#blue', '#green', '#blue']


def test_correct_hashtag_counts():
    assert hashtag_summary['hashtag_counts'] == [1, 2, 0]


def test_correct_hashtag_freq():
    assert hashtag_summary['hashtag_freq'] == [(0, 1), (1, 1), (2, 1)]


def test_correct_top_hashtags():
    assert hashtag_summary['top_hashtags'] == [('#blue', 2), ('#green', 1)]


def test_correct_hashtag_overview():
    hashtag_overview = hashtag_summary['overview']
    assert hashtag_overview['num_posts'] == 3
    assert hashtag_overview['num_hashtags'] == 3
    assert hashtag_overview['hashtags_per_post'] == 1.0
    assert hashtag_overview['unique_hashtags'] == 2


def test_emoji_result_has_correct_keys():
    assert set(emoji_summary.keys()) == set(emoji_test_keys)


def test_correct_emoji_extracted():
    assert emoji_summary['emoji'] == [['😀'], ['😀', '😉'], []]


def test_correct_flat_emoji():
    assert emoji_summary['emoji_flat'] == ['😀', '😀', '😉']


def test_correct_emoji_counts():
    assert emoji_summary['emoji_counts'] == [1, 2, 0]


def test_correct_emoji_freq():
    assert emoji_summary['emoji_freq'] == [(0, 1), (1, 1), (2, 1)]


def test_correct_top_emoji():
    assert emoji_summary['top_emoji'] == [('😀', 2), ('😉', 1)]


def test_correct_emoji_overview():
    emoji_overview = emoji_summary['overview']
    assert emoji_overview['num_posts'] == 3
    assert emoji_overview['num_emoji'] == 3
    assert emoji_overview['emoji_per_post'] == 1.0
    assert emoji_overview['unique_emoji'] == 2




import re
from itertools import permutations, combinations

import pandas as pd


def kw_generate(products, words, max_len=3,
                match_types=['Exact', 'Phrase', 'Modified'],
                order_matters=True, campaign_name='SEM_Campaign'):
    """Generate a data frame of keywords using a list of products and relevant
    words.

    :param products: will be used as the names of the ad groups
    :param words: related words that make it clear that the user is interested
        in ``products``
    :param max_len: the maximum number of words to include in each permutation
        of product keywords
    :param match_types: can be restricted or kept as is based on preference,
        possible values\: 'Exact', 'Phrase', 'Modified', 'Broad'
    :param order_matters: whether or not the order of words in keywords
        matters, default False
    :param campaign_name: name of campaign
    :returns keywords_df: a pandas.DataFrame ready to upload

    >>> import advertools as adv
    >>> products = ['bmw', 'toyota']
    >>> words = ['buy', 'second hand']
    >>> kw_df = adv.kw_generate(products, words)
    >>> kw_df.head()
           Campaign Ad Group          Keyword Criterion Type       Labels
    0  SEM_Campaign      Bmw          bmw buy          Exact          Buy
    1  SEM_Campaign      Bmw          bmw buy         Phrase          Buy
    2  SEM_Campaign      Bmw        +bmw +buy          Broad          Buy
    3  SEM_Campaign      Bmw  bmw second hand          Exact  Second Hand
    4  SEM_Campaign      Bmw  bmw second hand         Phrase  Second Hand

    >>> kw_df.tail()
            Campaign Ad Group                    Keyword Criterion Type           Labels
    55  SEM_Campaign   Toyota     second hand toyota buy         Phrase  Second Hand;Buy
    56  SEM_Campaign   Toyota  +second hand +toyota +buy          Broad  Second Hand;Buy
    57  SEM_Campaign   Toyota     second hand buy toyota          Exact  Second Hand;Buy
    58  SEM_Campaign   Toyota     second hand buy toyota         Phrase  Second Hand;Buy
    59  SEM_Campaign   Toyota  +second hand +buy +toyota          Broad  Second Hand;Buy
    """
    match_types = [x.title() for x in match_types]
    POSSIBLE_MATCH_TYPES = ['Exact', 'Phrase', 'Broad', 'Modified']
    if not set(match_types).issubset(POSSIBLE_MATCH_TYPES):
        raise ValueError('please make sure match types are any of '
                         + str(POSSIBLE_MATCH_TYPES))

    if max_len < 2:
        raise ValueError('please make sure max_len is >= 2')

    comb_func = permutations if order_matters else combinations
    headers = ['Campaign', 'Ad Group', 'Keyword', 'Criterion Type', 'Labels']
    keywords_list = []
    for prod in products:
        for i in range(2, max_len+1):
            for comb in comb_func([prod] + words, i):
                if prod not in comb:
                    continue
                for match in match_types:
                    row = [
                        campaign_name,
                        prod.title(),
                        (' '.join(comb) if match != 'Modified' else
                            '+' + ' '.join(comb).replace(' ', ' +')),
                        match if match != 'Modified' else 'Broad',
                        ';'.join([x.title() for x in comb if x != prod])
                    ]
                    keywords_list.append(row)
    return pd.DataFrame.from_records(keywords_list, columns=headers)


def kw_broad(words):
    """Return ``words`` in broad match.

    :param words: list of strings
    :returns formatted: `words` in broad match type

    >>> keywords = ['[learn guitar]', '"guitar courses"', '+guitar +tutor']
    >>> kw_broad(keywords)
    ['learn guitar', 'guitar courses', 'guitar tutor']
    """
    regex = r'^\'|^\"|\'$|\"$|\+|^\[|\]$|^-'
    return [re.sub(regex, '', x) for x in words]


def kw_exact(words):
    """Return ``words`` in exact match.

    :param words: list of strings
    :returns formatted: `words` in exact match type

    >>> keywords = ['learn guitar', 'guitar courses', 'guitar tutor']
    >>> kw_exact(keywords)
    ['[learn guitar]', '[guitar courses]', '[guitar tutor]']
    """
    return ['[' + w + ']' for w in kw_broad(words)]


def kw_phrase(words):
    """Return ``words`` in phrase match.

    :param words: list of strings
    :returns formatted: `words` in phrase match type

    >>> keywords = ['learn guitar', 'guitar courses', 'guitar tutor']
    >>> kw_phrase(keywords)
    ['"learn guitar"', '"guitar courses"', '"guitar tutor"']
    """
    return ['"' + w + '"' for w in kw_broad(words)]


def kw_modified(words):
    """Return ``words`` in modified broad match.

    :param words: list of strings
    :returns formatted: `words` in modified broad match type

    >>> keywords = ['learn guitar', 'guitar courses', 'guitar tutor']
    >>> kw_modified(keywords)
    ['+learn +guitar', '+guitar +courses', '+guitar +tutor']
    """
    return ['+' + w.replace(' ', ' +') for w in kw_broad(words)]


def kw_neg_broad(words):
    """Return ``words`` in negative broad match.

    :param words: list of strings
    :returns formatted: `words` in negative broad match type

    >>> keywords = ['learn guitar', 'guitar courses', 'guitar tutor']
    >>> kw_neg_broad(keywords)
    ['-learn guitar', '-guitar courses', '-guitar tutor']
    """
    return ['-' + w for w in kw_broad(words)]


def kw_neg_phrase(words):
    """Return ``words`` in negative phrase match.

    :param words: list of strings
    :returns formatted: `words` in negative phrase match type

    >>> keywords = ['learn guitar', 'guitar courses', 'guitar tutor']
    >>> kw_neg_phrase(keywords)
    ['-"learn guitar"', '-"guitar courses"', '-"guitar tutor"']
    """
    return ['-' + w for w in kw_phrase(words)]


def kw_neg_exact(words):
    """Return ``words`` in negative exact match.

    :param words: list of strings
    :returns formatted: `words` in negative exact match type

    >>> keywords = ['learn guitar', 'guitar courses', 'guitar tutor']
    >>> kw_neg_exact(keywords)
    ['-[learn guitar]', '-[guitar courses]', '-[guitar tutor]']
    """
    return ['-' + w for w in kw_exact(words)]
